/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  This class is an implementation of the Classifier with a boosting of
  trees. It works with samples from R^n and has no concept of the
  pi-features.

*/

#ifndef BOOSTED_CLASSIFIER_H
#define BOOSTED_CLASSIFIER_H

#include "classifier.h"
#include "sample_set.h"
#include "decision_tree.h"
#include "loss_machine.h"

class BoostedClassifier : public Classifier {
public:

  int _nb_weak_learners;
  DecisionTree **_weak_learners;

public:

  BoostedClassifier(int nb_weak_learners);
  BoostedClassifier();
  virtual ~BoostedClassifier();

  virtual scalar_t response(SampleSet *sample_set, int n_sample);
  virtual void train(LossMachine *loss_machine, SampleSet *train, scalar_t *response);

  virtual void tag_used_features(bool *used);
  virtual void re_index_features(int *new_indexes);

  virtual void read(istream *is);
  virtual void write(ostream *os);
};

#endif
