/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "gaussian.h"

Gaussian::Gaussian() {
  _nb_samples = 0;
  _sum = 0.0;
  _sum_sq = 0.0;
}

void Gaussian::add_sample(scalar_t x) {
  _nb_samples++;
  _sum += x;
  _sum_sq += x * x;
}

scalar_t Gaussian::expectation() {
  return _sum / scalar_t(_nb_samples);
}

scalar_t Gaussian::variance() {
  scalar_t e = _sum / scalar_t(_nb_samples);
  return (_sum_sq - _sum * e) / scalar_t(_nb_samples - 1);
}

scalar_t Gaussian::standard_deviation() {
  return sqrt(variance());
}

