#!/bin/bash

########################################################################
#                                                                      #
#  folded-ctf is an implementation of the folded hierarchy of	       #
#  classifiers for object detection, developed by Francois Fleuret     #
#  and Donald Geman.						       #
#								       #
#  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/   #
#  Written by Francois Fleuret <francois.fleuret@idiap.ch>	       #
#								       #
#  This file is part of folded-ctf.				       #
#								       #
#  folded-ctf is free software: you can redistribute it and/or modify  #
#  it under the terms of the GNU General Public License version 3 as   #
#  published by the Free Software Foundation.                          #
#								       #
#  folded-ctf is distributed in the hope that it will be useful, but   #
#  WITHOUT ANY WARRANTY; without even the implied warranty of	       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   #
#  General Public License for more details.			       #
#								       #
#  You should have received a copy of the GNU General Public License   #
#  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>. #
#								       #
########################################################################

GRAPH_NAME="/tmp/roc.eps"

#########################################################################

echo "Parsing the log files."

for p in hb h+b; do
    grep ^INFO results/${p}-*/log  | \
        awk '{
               if($2 == "DECIMATED_FALSE_NEGATIVE_RATE") {
                 printf(1-$3)
               } else if($2 == "DECIMATED_NB_FALSE_POSITIVES_PER_VGA") {
                 printf(" "$3"\n")
               }
             }' | sort -g > /tmp/${p}

    if [[ ! -s /tmp/${p} ]]; then
        echo "Not enough data points for ${p}." >&2
        exit 1
    fi
done

######################################################################

echo "Generating the graph per se."

gnuplot<<EOF
  set terminal postscript enhanced eps "Helvetica" 20
  set key 80,0.25
  set output "${GRAPH_NAME}"
  set logscale x
  set xlabel "Number of false alarms per 640x480"
  set ylabel "True positive rate"
  set grid

  plot [1e-3:100][0.0:1.0] \
     '/tmp/hb' using 2:1 title "HB" pt 7 ps 1.0 lc 1 lw 1,\
     '/tmp/h+b' using 2:1 title "H+B" pt 7 ps 1.0 lc 3 lw 1
EOF

######################################################################

echo "Graph saved in ${GRAPH_NAME}"
