/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "image.h"

Image::Image(int width, int height) {
  _width = width;
  _height = height;
  _content = new unsigned char[_width * _height];
}

Image::Image() {
  _width = 0;
  _height = 0;
  _content = 0;
}

Image::~Image() {
  delete[] _content;
}

void Image::to_rgb(RGBImage *image) {
  int c;
  for(int y = 0; y < _height; y++) {
    for(int x = 0; x < _width; x++) {
      c = value(x, y);
      image->set_pixel(x, y, c, c, c);
    }
  }
}

void Image::read(istream *in) {
  delete[] _content;
  read_var(in, &_width);
  read_var(in, &_height);
  _content = new unsigned char[_width * _height];
  in->read((char *) _content, sizeof(unsigned char) * _width * _height);
}

void Image::write(ostream *out) {
  write_var(out, &_width);
  write_var(out, &_height);
  out->write((char *) _content, sizeof(unsigned char) * _width * _height);
}
