/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  An implementation of a gray-scale image with a minimum set of
  methods.

 */

#ifndef IMAGE_H
#define IMAGE_H

#include "storable.h"
#include "rgb_image.h"
#include "misc.h"

class Image : public Storable {
protected:
  int _width, _height;
  unsigned char *_content;
public:

  inline int width() { return _width; }
  inline int height() { return _height; }

  inline unsigned char value(int x, int y) {
    if(x >= 0 && x < _width && y >= 0 && y < _height)
      return _content[x + (y * _width)];
    else
      return 0;
  }

  inline void set_value(int x, int y, unsigned char v) {
    if(x >= 0 && x < _width && y >= 0 && y < _height)
      _content[x + (y * _width)] = v;
    else abort();
  }

  Image();
  Image(int width, int height);

  virtual ~Image();

  virtual void to_rgb(RGBImage *image);

  virtual void read(istream *in);
  virtual void write(ostream *out);
};

#endif
