/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "interval.h"

void Interval::set(scalar_t x) {
  min = x;
  max = x;
}

void Interval::set(scalar_t a, scalar_t b) {
  min = a;
  max = b;
}

void Interval::set(Interval *i) {
  min = i->min;
  max = i->max;
}

void Interval::set_subinterval(Interval *i, int k, int nb) {
  min = i->min + ((i->max - i->min) * scalar_t(k))/scalar_t(nb);
  max = i->min + ((i->max - i->min) * scalar_t(k + 1))/scalar_t(nb);
}

void Interval::swallow(Interval *i) {
  min = ::min(min, i->min);
  max = ::max(max, i->max);
}

ostream &operator << (ostream &out, const Interval &i) {
  return out << "[" << i.min << ", " << i.max << "]";
}
