/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  An interval class.

 */

#ifndef INTERVAL_H
#define INTERVAL_H

#include "misc.h"

class Interval {
public:
  scalar_t min, max;

  void set(scalar_t x);
  void set(scalar_t a, scalar_t b);
  void set(Interval *i);

  // Set this interval to the k-th of the nb regular subintervals of i
  void set_subinterval(Interval *i, int k, int nb);

  // Grow to contain i
  void swallow(Interval *i);

  inline bool contains(scalar_t x) {
    return x >= min && x < max;
  }

  inline scalar_t middle() {
    return (min + max) / 2;
  }
};

ostream &operator << (ostream &out, const Interval &i);

#endif
