/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "labelled_image.h"

LabelledImage::LabelledImage() : RichImage() {
  _target_poses = 0;
}

LabelledImage::LabelledImage(int width, int height, int nb_targets) : RichImage(width, height) {
  _nb_targets = nb_targets;
  _target_poses = new Pose[_nb_targets];
}

LabelledImage::~LabelledImage() {
  delete[] _target_poses;
}

int LabelledImage::pose_cell_label(PoseCell *cell) {
  int positive = 0;
  int negative = 1;

  for(int t = 0; t < _nb_targets; t++) {
    if(cell->contains(_target_poses + t))
      positive = 1;
    if(!cell->negative_for_train(_target_poses + t))
      negative = 0;
  }

  if(positive) return 1;
  if(negative) return -1;
  return 0;
}

void LabelledImage::write(ostream *out) {
  int v = file_format_version;
  write_var(out, &v);
  RichImage::write(out);
  write_var(out, &_nb_targets);
  for(int t = 0; t < _nb_targets; t++)
    _target_poses[t].write(out);
}

void LabelledImage::read(istream *in) {
  int v;
  read_var(in, &v);
  if(v != file_format_version) {
    cerr << "Pool file format version " << file_format_version << " expected,"
         << " the file is version " << v
         << endl;
    exit(1);
  }
  RichImage::read(in);
  delete[] _target_poses;
  read_var(in, &_nb_targets);
  _target_poses = new Pose[_nb_targets];
  for(int t = 0; t < _nb_targets; t++)
    _target_poses[t].read(in);
}
