/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  A LabelledImage is a RichImage labelled with the ground truth,
  i.e. the target poses.

 */

#ifndef LABELLED_IMAGE_H
#define LABELLED_IMAGE_H

#include "rich_image.h"
#include "pose.h"
#include "pose_cell.h"

class LabelledImage : public RichImage {
  int _nb_targets;
  Pose *_target_poses;

  static const int file_format_version = 2;

public:

  LabelledImage();
  LabelledImage(int width, int height, int nb_targets);

  virtual ~LabelledImage();

  inline int nb_targets() { return _nb_targets; }
  inline Pose *get_target_pose(int n) { return _target_poses + n; }

  // The label of a cell can be +1 if it contains a target and -1 if
  // it is far enough from any target
  int pose_cell_label(PoseCell *cell);

  virtual void write(ostream *out);
  virtual void read(istream *in);
};

#endif
