/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  A pool of images is used to avoid having too many images in memory
  at the same time. It can be either a file, or a subset of another
  pool of images (see the subclasses).

 */

#ifndef LABELLED_IMAGE_POOL_H
#define LABELLED_IMAGE_POOL_H

#include "shared.h"
#include "labelled_image.h"

class LabelledImagePool {
public:
  virtual ~LabelledImagePool();
  virtual int nb_images() = 0;
  // grab_image _DOES NOT_ build the rich structure of the grabbed
  // image. One has to call compute_rich_structure() explicitely for
  // that!
  virtual LabelledImage *grab_image(int n_image) = 0;
  virtual void release_image(int n_image) = 0;
};

#endif
