/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "labelled_image_pool_subset.h"

LabelledImagePoolSubset::LabelledImagePoolSubset(LabelledImagePool *mother_pool,
                                                 bool *used) {
  _mother_pool = mother_pool;
  _nb_images = 0;
  for(int n = 0; n < _mother_pool->nb_images(); n++)
    if(used[n]) _nb_images++;
  _indexes = new int[_nb_images];
  int k = 0;
  for(int n = 0; n < _mother_pool->nb_images(); n++)
    if(used[n]) _indexes[k++] = n;
}

LabelledImagePoolSubset::~LabelledImagePoolSubset() {
  delete[] _indexes;
}

int LabelledImagePoolSubset::nb_images() {
  return _nb_images;
}

LabelledImage *LabelledImagePoolSubset::grab_image(int n_image) {
  return _mother_pool->grab_image(_indexes[n_image]);
}

void LabelledImagePoolSubset::release_image(int n_image) {
  _mother_pool->release_image(_indexes[n_image]);
}
