/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  A ParsingPool is a family of Parsings associated to all the images
  of a LabelledImagePool.

*/

#ifndef PARSING_POOL_H
#define PARSING_POOL_H

#include "parsing.h"
#include "pi_feature_family.h"
#include "classifier.h"
#include "labelled_image_pool.h"
#include "pose_cell_hierarchy.h"

class ParsingPool {
  int _nb_images;
  long int _nb_cells, _nb_positive_cells, _nb_negative_cells;
  Parsing **_parsings;

public:

  inline int nb_cells() {
    return _nb_cells;
  }

  inline int nb_positive_cells() {
    return _nb_positive_cells;
  }

  inline int nb_negative_cells() {
    return _nb_negative_cells;
  }

  ParsingPool(LabelledImagePool *image_pool, PoseCellHierarchy *hierarchy, scalar_t proportion_negative_cells);

  ~ParsingPool();

  // The parameter level is the resulting level, not the starting one,
  // hence should always be strictly positive
  void down_one_level(LossMachine *loss_machine, PoseCellHierarchy *hierarchy, int level);

  void update_cell_responses(PiFeatureFamily *pi_feature_family,
                             Classifier *classifier);

  // Collect all positive and sample negative examples
  void weighted_sampling(LossMachine *loss_machine,
                         PiFeatureFamily *pi_feature_family,
                         SampleSet *sample_set,
                         scalar_t *responses);

  void write_roc(ofstream *out);
};

#endif
