/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  A PiFeatures is the central new idea of this approach. It is a
  feature which can be evaluated on a pair image / referential, where
  the referential is computed from a pose cell.

*/

#ifndef PI_FEATURE_H
#define PI_FEATURE_H

#include "misc.h"
#include "global.h"
#include "rich_image.h"
#include "pi_referential.h"

class PiFeature {

  enum {
    PF_EDGE_THRESHOLDING,
    PF_EDGE_HISTOGRAM_COMPARISON,
    PF_GRAYSCALE_HISTOGRAM_COMPARISON,
  } _type;

  int _tag, _edge_scale;
  Rectangle _window_a, _window_b;
  int _registration_a, _registration_b;

  int random_registration_mode(int level);
  void randomize_window(int registration_mode, Rectangle *window);
  void draw_window(RGBImage *image, int registration_mode, Rectangle *window);

  // EDGE THRESHOLDING

  scalar_t response_edge_thresholding(RichImage *image, PiReferential *referential);
  void draw_edge_thresholding(RGBImage *image, int r, int g, int b,
                              PiReferential *referential);
  void print_edge_thresholding(ostream *os);

  // EDGE ORIENTATION HISTOGRAM COMPARISON

  scalar_t response_edge_histogram_comparison(RichImage *image, PiReferential *referential);
  void draw_edge_histogram_comparison(RGBImage *image, int r, int g, int b,
                                      PiReferential *referential);
  void print_edge_histogram_comparison(ostream *os);

  // GRAYSCALE HISTOGRAM COMPARISON

  scalar_t response_grayscale_histogram_comparison(RichImage *image, PiReferential *referential);
  void draw_grayscale_histogram_comparison(RGBImage *image,
                                           int r, int g, int b,
                                           PiReferential *referential);
  void print_grayscale_histogram_comparison(ostream *os);

public:

  void randomize(int level);

  scalar_t response(RichImage *image, PiReferential *referential);

  void draw(RGBImage *image,
            int r, int g, int b,
            PiReferential *referential);

  void print(ostream *os);
};

#endif
