/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "pi_feature_family.h"

PiFeatureFamily::PiFeatureFamily() {
  _nb_features = 0;
  _pi_features = 0;
}

PiFeatureFamily::~PiFeatureFamily() {
  delete[] _pi_features;
}

void PiFeatureFamily::read(istream *is) {
  delete[] _pi_features;
  read_var(is, &_nb_features);
  _pi_features = new PiFeature[_nb_features];
  is->read((char *) _pi_features, sizeof(PiFeature) * _nb_features);
}

void PiFeatureFamily::write(ostream *os) {
  write_var(os, &_nb_features);
  os->write((char *) _pi_features, sizeof(PiFeature) * _nb_features);
}

void PiFeatureFamily::resize(int nb_features) {
  delete[] _pi_features;
  _nb_features = nb_features;
  _pi_features = new PiFeature[_nb_features];
}

void PiFeatureFamily::randomize(int level) {
  for(int f = 0; f < _nb_features; f++) _pi_features[f].randomize(level);
}

void PiFeatureFamily::extract(PiFeatureFamily *pi_feature_family,
                              bool *used_features, int *new_feature_indexes) {
  delete[] _pi_features;
  _nb_features = 0;

  for(int f = 0; f < pi_feature_family->nb_features(); f++)
    if(used_features[f]) _nb_features++;

  _pi_features = new PiFeature[_nb_features];

  int g = 0;

  for(int f = 0; f < pi_feature_family->nb_features(); f++)
    if(used_features[f]) {
      _pi_features[g] = pi_feature_family->_pi_features[f];
      new_feature_indexes[f] = g;
      g++;
    } else new_feature_indexes[f] = -1;
}
