/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  A PiFeatureFamily is simply a set of pi-features, with a method to
  extract a subfamily, so that we can forget all the features not used
  by a classifier when the learning is over.

 */

#ifndef PI_FEATURE_FAMILY_H
#define PI_FEATURE_FAMILY_H

#include "misc.h"
#include "pose_cell_set.h"
#include "pi_feature.h"

class PiFeatureFamily : public Storable {
  int _nb_features;
  PiFeature *_pi_features;

public:
  PiFeatureFamily();
  ~PiFeatureFamily();

  inline int nb_features() { return _nb_features; }

  inline PiFeature *get_feature(int f) {
    ASSERT(f >= 0 && f < _nb_features);
    return _pi_features + f;
  }

  void read(istream *is);
  void write(ostream *os);

  void resize(int nb_features);
  void randomize(int level);

  void extract(PiFeatureFamily *pi_feature_family, bool *used_features, int *new_feature_indexes);
};

#endif
