/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  This the pose of a target. In the case of cats, we have a bounding
  box, the head location and scale, and the body location. All are
  given in the image coordinate system.

 */

#ifndef POSE_H
#define POSE_H

#include "rgb_image.h"
#include "image.h"

class Pose {
public:
  scalar_t _bounding_box_xmin, _bounding_box_ymin;
  scalar_t _bounding_box_xmax, _bounding_box_ymax;
  scalar_t _head_xc, _head_yc, _head_radius;
  scalar_t _belly_xc, _belly_yc;

  Pose();

  bool hit(int level, Pose *pose);
  bool collide(int level, Pose *pose);

  void horizontal_flip(scalar_t scene_width);

  void draw(int thickness, int r, int g, int b, int level, RGBImage *image);

  void print(ostream *out);

  void write(ostream *out);
  void read(istream *in);
};

#endif
