/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "pose_cell.h"
#include "global.h"
#include "tools.h"

bool PoseCell::contains(Pose *pose) {
  return
    _head_xc.contains(pose->_head_xc) &&
    _head_yc.contains(pose->_head_yc) &&
    _head_radius.contains(pose->_head_radius) &&
    _belly_xc.contains(pose->_belly_xc) &&
    _belly_yc.contains(pose->_belly_yc);
}

bool PoseCell::negative_for_train(Pose *pose) {
  const scalar_t r_tolerance = 3;
  scalar_t r = pose->_head_radius;
  return
    r <= _head_radius.min / r_tolerance ||
    r >= _head_radius.max * r_tolerance ||
    pose->_head_xc <= _head_xc.min - r ||
    pose->_head_xc >= _head_xc.max + r ||
    pose->_head_yc <= _head_yc.min - r ||
    pose->_head_yc >= _head_yc.max + r;
}

void PoseCell::get_centroid(Pose *pose) {
  pose->_bounding_box_xmin = -1;
  pose->_bounding_box_ymin = -1;
  pose->_bounding_box_xmax = -1;
  pose->_bounding_box_ymax = -1;
  pose->_head_radius = sqrt(_head_radius.min * _head_radius.max);
  pose->_head_xc = _head_xc.middle();
  pose->_head_yc = _head_yc.middle();
  pose->_belly_xc = _belly_xc.middle();
  pose->_belly_yc = _belly_yc.middle();
}

void PoseCell::print(ostream *out) {
  (*out) << "  _head_xc " << _head_xc << endl;
  (*out) << "  _head_yc " << _head_yc << endl;
  (*out) << "  _head_radius " << _head_radius << endl;
  (*out) << "  _head_tilt " << _head_tilt << endl;
  (*out) << "  _belly_xc " << _belly_xc << endl;
  (*out) << "  _belly_yc " << _belly_yc << endl;
}
