/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  A PoseCell is a 5d rectangle in the pose space.

 */

#ifndef POSE_CELL_H
#define POSE_CELL_H

#include "pose.h"
#include "interval.h"

class PoseCell {
public:

  Interval _head_xc, _head_yc, _head_radius, _head_tilt;
  Interval _belly_xc, _belly_yc;

  // The cell contains the pose

  bool contains(Pose *pose);

  // The pose is far enough from the cell to accept the cell as a
  // negative sample

  bool negative_for_train(Pose *pose);

  // Copies into pose the average pose of that cell

  void get_centroid(Pose *pose);

  void print(ostream *out);
};

#endif
