/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  A PoseCellHierarchy provides the necessary methods to visit a
  recursive partitioning of the pose space. The two main methods can
  give you an root partitioning, or compute a finer partitioning from
  an existing one.

 */

#ifndef POSE_CELL_HIERARCHY_H
#define POSE_CELL_HIERARCHY_H

#include "pose_cell_set.h"
#include "labelled_image_pool.h"

struct RelativeBellyPoseCell {
  Interval _belly_xc, _belly_yc;
};

class PoseCellHierarchy {
  static const scalar_t pseudo_infty = 10000;

  static const scalar_t belly_resolution = 0.5;

  static const int nb_radius_1 = 16;
  static const int nb_radius_2 = 16;
  static const int nb_tilts = 64;

  int _nb_levels;
  scalar_t _min_head_radius;
  scalar_t _max_head_radius;
  int _root_cell_nb_xy_per_radius;

  int _nb_belly_cells;

  RelativeBellyPoseCell *_belly_cells;

public:
  PoseCellHierarchy();
  PoseCellHierarchy(LabelledImagePool *train_pool);
  virtual ~PoseCellHierarchy();

  virtual int nb_levels();
  virtual void get_containing_cell(Image *image, int level,
                                   Pose *pose, PoseCell *result_cell);

  virtual void add_root_cells(Image *image, PoseCellSet *cell_set);
  // level is the level to build, hence should be greater than 1
  virtual void add_subcells(int level, PoseCell *root, PoseCellSet *cell_set);

  virtual int nb_incompatible_poses(LabelledImagePool *pool);

  virtual void write(ostream *os);
  virtual void read(istream *is);
};

#endif
