/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  A set of PoseCells each paired with a detector response. This is
  what the parsing of a scene produces.

 */

#ifndef POSE_CELL_SCORED_SET_H
#define POSE_CELL_SCORED_SET_H

#include "pose_cell.h"
#include "labelled_image.h"

#include "pose_cell_set.h"

class PoseCellScoredSet : public PoseCellSet {
  scalar_t *_scores;

public:

  PoseCellScoredSet();
  ~PoseCellScoredSet();

  inline scalar_t get_score(int k) {
    ASSERT(k >= 0 && k < _nb_added);
    return _scores[k];
  }

  void add_cell_with_score(PoseCell *cell, scalar_t score);

  void decimate_hit(int level);
  void decimate_collide(int level);
};

#endif
