/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "pose_cell_set.h"
#include "global.h"

PoseCellSet::PoseCellSet() {
  _nb_added = 0;
  _max_nb = max_nb_cells;
  _cells = new PoseCell[_max_nb];
}

PoseCellSet::~PoseCellSet() {
  delete[] _cells;
}

void PoseCellSet::erase_content() {
  _nb_added = 0;
}

PoseCell *PoseCellSet::get_containing_cell(Pose *p) {
  for(int c = 0; c < _nb_added; c++) {
    if(_cells[c].contains(p)) return _cells + c;
  }
  return 0;
}

void PoseCellSet::add_cell(PoseCell *cell) {
  if(_nb_added < _max_nb) {
    _cells[_nb_added] = *cell;
    _nb_added++;
  } else {
    cerr << "Too many pose cells!" << endl;
    abort();
  }
}
