/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  A set of PoseCell.

 */

#ifndef POSE_CELL_SET_H
#define POSE_CELL_SET_H

#include "pose_cell.h"
#include "labelled_image.h"

class PoseCellSet {
protected:
  // This is a bit ugly. Notice that all the code is written in such a
  // way that we never have more than one such PoseCellSet in memory
  // at any moment
  static const int max_nb_cells = 500000;

  int _max_nb;
  int _nb_added;
  PoseCell *_cells;

public:

  PoseCellSet();
  ~PoseCellSet();

  inline int nb_cells() { return _nb_added; }

  inline PoseCell *get_cell(int k) {
    ASSERT(k >= 0 && k < _nb_added);
    return _cells + k;
  }

  PoseCell *get_containing_cell(Pose *p);

  void erase_content();
  void add_cell(PoseCell *cell);
};

#endif
