/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "rgb_image_subpixel.h"

RGBImageSubpixel::RGBImageSubpixel(int width, int height) : RGBImage(width * scale, height* scale) { }

RGBImageSubpixel::RGBImageSubpixel(RGBImage *image) : RGBImage(image->width() * scale, image->height() * scale) {
  for(int y = 0; y < _height; y++) {
    for(int x = 0; x < _width; x++) {
      set_pixel(x, y,
                image->pixel(x / scale, y / scale, RGBImage::RED),
                image->pixel(x / scale, y / scale, RGBImage::GREEN),
                image->pixel(x / scale, y / scale, RGBImage::BLUE));
    }
  }
}

RGBImageSubpixel::~RGBImageSubpixel() { }

void RGBImageSubpixel::read_png(const char *filename) {
  abort();
}

void RGBImageSubpixel::write_png(const char *filename) {
  RGBImage tmp(_width / scale, _height / scale);
  for(int y = 0; y < _height / scale; y++) {
    for(int x = 0; x < _width / scale; x++) {
      int sr = 0, sg = 0, sb = 0;
      for(int yy = y * scale; yy < (y + 1) * scale; yy++) {
        for(int xx = x * scale; xx < (x + 1) * scale; xx++) {
          sr += int(_bit_plans[RED][yy][xx]);
          sg += int(_bit_plans[GREEN][yy][xx]);
          sb += int(_bit_plans[BLUE][yy][xx]);
        }
      }

      tmp.set_pixel(x, y,
                    sr / (scale * scale), sg / (scale * scale), sb / (scale * scale));
    }
  }
  tmp.write_png(filename);
}


void RGBImageSubpixel::read_jpg(const char *filename) {
  abort();
}

void RGBImageSubpixel::write_jpg(const char *filename, int quality) {
  RGBImage tmp(_width / scale, _height / scale);

  for(int y = 0; y < _height / scale; y++) {
    for(int x = 0; x < _width / scale; x++) {
      int sr = 0, sg = 0, sb = 0;
      for(int yy = y * scale; yy < (y + 1) * scale; yy++) {
        for(int xx = x * scale; xx < (x + 1) * scale; xx++) {
          sr += int(_bit_plans[RED][yy][xx]);
          sg += int(_bit_plans[GREEN][yy][xx]);
          sb += int(_bit_plans[BLUE][yy][xx]);
        }
      }

      tmp.set_pixel(x, y,
                    sr / (scale * scale), sg / (scale * scale), sb / (scale * scale));
    }
  }

  tmp.write_jpg(filename, quality);
}


void RGBImageSubpixel::draw_line(int thickness,
                                 unsigned char r, unsigned char g, unsigned char b,
                                 scalar_t x0, scalar_t y0, scalar_t x1, scalar_t y1) {
  RGBImage::draw_line(int(thickness * scale),
                      r, g, b,
                      x0 * scale + scale/2, y0 * scale + scale/2,
                      x1 * scale + scale/2, y1 * scale + scale/2);
}
