/*
 *  folded-ctf is an implementation of the folded hierarchy of
 *  classifiers for object detection, developed by Francois Fleuret
 *  and Donald Geman.
 *
 *  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of folded-ctf.
 *
 *  folded-ctf is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  folded-ctf is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*

  Same as RGBImage, but with sub-pixel resolution to produce nifty
  smoothing in the drawings.

 */

#ifndef RGB_IMAGE_SUBPIXEL_H
#define RGB_IMAGE_SUBPIXEL_H

#include "misc.h"
#include "rgb_image.h"

class RGBImageSubpixel : public RGBImage {
  static const int scale = 8;
public:

  RGBImageSubpixel(int width, int height);
  RGBImageSubpixel(RGBImage *image);
  virtual ~RGBImageSubpixel();

  inline int width() const { return _width / scale; }
  inline int height() const { return _height / scale; }

  virtual void read_png(const char *filename);
  virtual void write_png(const char *filename);

  virtual void read_jpg(const char *filename);
  virtual void write_jpg(const char *filename, int quality);

  virtual void draw_line(int thickness,
                         unsigned char r, unsigned char g, unsigned char b,
                         scalar_t x0, scalar_t y0, scalar_t x1, scalar_t y1);
};

#endif
