#!/bin/bash


########################################################################
#                                                                      #
#  folded-ctf is an implementation of the folded hierarchy of	       #
#  classifiers for object detection, developed by Francois Fleuret     #
#  and Donald Geman.						       #
#								       #
#  Copyright (c) 2008 Idiap Research Institute, http://www.idiap.ch/   #
#  Written by Francois Fleuret <francois.fleuret@idiap.ch>	       #
#								       #
#  This file is part of folded-ctf.				       #
#								       #
#  folded-ctf is free software: you can redistribute it and/or modify  #
#  it under the terms of the GNU General Public License version 3 as   #
#  published by the Free Software Foundation.                          #
#								       #
#  folded-ctf is distributed in the hope that it will be useful, but   #
#  WITHOUT ANY WARRANTY; without even the implied warranty of	       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   #
#  General Public License for more details.			       #
#								       #
#  You should have received a copy of the GNU General Public License   #
#  along with folded-ctf.  If not, see <http://www.gnu.org/licenses/>. #
#								       #
########################################################################

MAIN_URL="http://www.idiap.ch/folded-ctf/"

#########################################################################
# Compiling

make -j -k

if [[ $? != 0 ]]; then
    echo "Compilation failed." >&2
    exit 1
fi

echo

#########################################################################
# Generating the pool file

DATA_PATH=./rmk-data
POOL_NAME=${DATA_PATH}/rmk.pool

if [[ -d ${DATA_PATH} ]]; then

    if [[ -f ${POOL_NAME} ]]; then

        echo "The pool file exists."

    else

        echo "Can not find the pool file, checking the data integrity."

        md5sum -c ${DATA_PATH}/list.md5

        if [[ $? != 0 ]]; then
            echo "The data set is corrupted. You can download it" >&2
            echo "from ${MAIN_URL}" >&2
            exit 1
        fi

        echo "Generating the pool file."

        ./list_to_pool ${DATA_PATH}/full.lst ${DATA_PATH} ${POOL_NAME}.wrk

        if [[ $? == 0 ]]; then
            mv ${POOL_NAME}.wrk ${POOL_NAME}
        else
            \rm ${POOL_NAME}.wrk 2> /dev/null
            echo "Pool generation failed." >&2
            exit 1
        fi

    fi

else

    echo "Can not find the RateMyKitten images in ${DATA_PATH}. You can download" >&2
    echo "them from ${MAIN_URL}" >&2
    exit 1

fi

RESULT_DIR=./results

case $1 in

    #####################################################################
    ## Generate illustrating pictures

    pics)

        SEED=0

        EXPERIMENT_RESULT_DIR="${RESULT_DIR}/hb-${SEED}"

        if [[ -d "${EXPERIMENT_RESULT_DIR}" ]]; then

            # Value of -1 corresponds to saving the images with the
            # ground-truth or the pi-referentials alone, while other
            # values show registered pi-feature windows.

            for npf in -1 0 2500; do

                ./folding --random-seed=${SEED} \
                    --pool-name=${POOL_NAME} \
                    --result-path=/tmp/ \
                    --detector-name=${EXPERIMENT_RESULT_DIR}/default.det \
                    --nb-images=63 \
                    --material-feature-nb=${npf} \
                    open-pool \
                    read-detector \
                    write-pool-images

            done

        else
            echo "You have to run at least the first round completely to be able" >&2
            echo "to generate the pictures." >&2
            exit 1
        fi

        ;;

    #####################################################################
    ## Run the full computation

    valgrind|"")

        if [[ ! -d ${RESULT_DIR} ]]; then
            mkdir ${RESULT_DIR}
        fi

        for SEED in {0..9}; do

            for MODE in hb h+b; do

                EXPERIMENT_RESULT_DIR="${RESULT_DIR}/${MODE}-${SEED}"

                mkdir ${EXPERIMENT_RESULT_DIR} 2> /dev/null

                if [[ $? == 0 ]]; then

                    if [[ $MODE == "h+b" ]]; then
                        OPTS="--force-head-belly-independence=yes"
                    else
                        OPTS=""
                    fi

                    if [[ $1 == "valgrind" ]]; then
                        # The valgrind operation runs a simpler computation in valgrind
                        OPTS="${OPTS} --nb-classifiers-per-level=1 --nb-weak-learners-per-classifier=10"
                        OPTS="${OPTS} --proportion-for-train=0.1 --proportion-for-validation=0.025 --proportion-for-test=0.01"
                        OPTS="${OPTS} --wanted-true-positive-rate=0.1"
                        DEBUGGER="valgrind --db-attach=yes --leak-check=full --show-reachable=yes"
                    fi

                    ${DEBUGGER} ./folding \
                        --random-seed=${SEED} \
                        --pool-name=${POOL_NAME} \
                        --result-path=${EXPERIMENT_RESULT_DIR} \
                        --detector-name=${EXPERIMENT_RESULT_DIR}/default.det \
                        ${OPTS} \
                        open-pool \
                        train-detector \
                        compute-thresholds \
                        write-detector \
                        sequence-test-detector | tee -a ${EXPERIMENT_RESULT_DIR}/stdout

                else

                    echo "${EXPERIMENT_RESULT_DIR} exists, cancelling that run."

                fi

            done

        done

esac
